/*
    Release 1.01  June 15, 2018, Jack Purdum, W8TEE. Minor edits to the code to make it easier to read
    
    Release 1.0   Oct.13, 2017, Jack Purdum, W8TEE, Al Peter, AC8GY. You may use/modify this code,
                  but keep this comment intact in the file and place any changes in from of this
                  entry.
*/

#include <Wire.h>               // Standard IDE library file
#include <Adafruit_GFX.h>       // https://github.com/adafruit/Adafruit-GFX-Library
#include <Adafruit_SSD1306.h>   // https://github.com/adafruit/Adafruit_SSD1306

//#define DEBUG                       // Uncomment to calibrate and debug

#define OLED_RESET               4
#define SENSORPIN               A1          // pin that the sensor is attached to
#define ITERATIONS              30

#define MYDUMMYLOADOHMS         50.1    	  // This needs to be changed for yours. See article in QST
#define CALIBRATIONOFFSET       0.008704    // This, too.
#define DIODEVOLTAGEDROP        0.7     	  // Voltage drop from diode

Adafruit_SSD1306 display(OLED_RESET);

int sensorValue = 0;         // the sensor value
int sensorMin = 1023;        // minimum sensor value
int sensorMax = 0;           // maximum sensor value


/*****
  Purpose: A simple splash sceen to show who developed the DL. Feel free to
           change to whatever your want.\, but comment out the old code and
           replace with your code.
           
  Parameter list:
    void
    
  Return value:
    void
    
*****/
void Splash()
{
  display.clearDisplay();     // Clear the graphics display buffer.
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(0, 0);
  display.println(" GCBG DUMMY LOAD");
  display.println("       by");
  display.println(" Al Peter, AC8GY");
  display.println("Jack Purdum, W8TEE");
  display.display();
  /*  If you want to use your name and call on the splash
      screen, comment out the above (don't remove it) and
      replace with something like this:
  display.setCursor(0, 1);
  display.println(" GCBG DUMMY LOAD");
  display.println("    built by");
  display.println("Steve Ford, W1AW");
  display.display();
   */
}


void setup()   {
#ifdef DEBUG  
  Serial.begin(9600);
#endif

  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3C (for the 128x32). 
                                              // Your OLED might use a different frequency. Google
                                              // "Arduino I2C scanner" or look at:
                                              // http://playground.arduino.cc/Main/I2cScanner

  Splash();

#ifdef DEBUG                  // Probably only need to call once to tweak things. Uncomment DEBUG line above
  Calibrate();
#endif  
  delay(4000);
  display.clearDisplay();     // Clear the graphics display buffer.
}


void loop() {
  char buff[10];
  char pad[] = "       ";    // 7 spaces
  int where;
  int i;
  float sum;
  float watts;

  sum = 0.0;
  i = 0;
  while (i < ITERATIONS) {
    sensorValue = analogRead(SENSORPIN);    // Input from voltage divider
    delay(10);                              // Let pin settle
    watts = CalculateWatts(sensorValue);
    sum += watts;
    i++;
  }
  sum /= ITERATIONS;
  dtostrf(sum, 4, 2, buff);

  where = strlen(buff);
  strcpy(&pad[6 - where], buff);
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(0, 0);
  display.println("      WATTS IN");
  display.setTextSize(3);
  where = (6 - where) * 10;
  display.setCursor(where, 10);
  display.println(buff);
  display.display();
  delay(1000);

}

/*****
  Purpose: To convert the value read from the DL and convert it to watts

  Parameter list:
    int sensorValue       the value read on the analog input pin

  Return value:
    float               the calculated watts
*****/
float CalculateWatts(int sensorValue)
{
  float temp;

  temp = sensorValue + DIODEVOLTAGEDROP;  // add back diode voltage drop
  temp = sensorValue;
  temp *= temp;                           // Square for power
  temp /= MYDUMMYLOADOHMS;                // Replace with your measured resistance

  return temp * CALIBRATIONOFFSET;
}

/*****
  Purpose: To figure out calibration constant, See book for details.

  Parameter list:
    void

  Return value:
    void
*****/

void Calibrate()
{
  int i = ITERATIONS;                      // We're going to get a series of readings
  
  while (i--) {
    sensorValue = analogRead(SENSORPIN);
    delay(100);                           // Settle between readings...
    if (sensorValue > sensorMax) {        // record the maximum sensor value
      sensorMax = sensorValue;
    }
    if (sensorValue < sensorMin) {        // record the minimum sensor value
      sensorMin = sensorValue;
    }
  }
#ifdef DEBUG                              // Only show if DEBUG mode set
    Serial.print("sensorValue = ");
    Serial.print(sensorValue);
    Serial.print("   sensorMin = ");
    Serial.println(sensorMin);
#endif
}


